"""Utility code for working with Concrete
"""

from concrete_uuid import *
from file_io import *
from redis_io import *
from json_fu import *
from references import *
from tokenization import *
from unnone import *



import logging

from concrete.structure.ttypes import TokenizationKind


def get_tokens(tokenization, suppress_warnings=False):
    '''
    Return list of Tokens from lattice.cachedBestPath, if Tokenization
    kind is TOKEN_LATTICE; else, return list of Tokens from tokenList.

    Warn and return list of Tokens from tokenList if kind is not set.
    Return None if kind is set but the respective data fields are not.
    '''

    if tokenization.kind is None:
        if not suppress_warnings:
            logging.warn('Tokenization.kind is None but is required in Concrete; using tokenList')

        token_list = tokenization.tokenList
        if token_list.tokenList is not None:
            return token_list.tokenList

    elif tokenization.kind == TokenizationKind.TOKEN_LATTICE:
        token_lattice = tokenization.lattice
        if token_lattice.cachedBestPath is not None:
            lattice_path = token_lattice.cachedBestPath
            if lattice_path.tokenList is not None:
                return lattice_path.tokenList

    elif tokenization.kind == TokenizationKind.TOKEN_LIST:
        token_list = tokenization.tokenList
        if token_list.tokenList is not None:
            return token_list.tokenList

    else:
        raise ValueError('Unrecognized TokenizationKind %d' % tokenization.kind)

    return None


def get_lemmas(tokenization):
    '''
    Return list of lemmas (as TaggedTokens), if there is a unique
    choice.

    Raise exception if there is no lemma tagging or more than one
    lemma tagging.
    '''

    lemma_tts = [
        tt
        for tt in tokenization.tokenTaggingList
        if tt.taggingType == u'LEMMA'
    ]
    if len(lemma_tts) == 0:
        raise Exception('No lemma tagging.')
    elif len(lemma_tts) == 1:
        return lemma_tts[0].taggedTokenList
    else:
        raise Exception('More than one lemma tagging.')


def get_pos(tokenization):
    '''
    Return list of POS tags (as TaggedTokens), if there is a unique
    choice.

    Raise exception if there is no POS tagging or more than one
    POS tagging.
    '''

    pos_tts = [
        tt
        for tt in tokenization.tokenTaggingList
        if tt.taggingType == u'POS'
    ]
    if len(pos_tts) == 0:
        raise Exception('No POS tagging.')
    elif len(pos_tts) == 1:
        return pos_tts[0].taggedTokenList
    else:
        raise Exception('More than one POS tagging.')


plus = lambda x, y: x + y

flatten = lambda a: reduce(plus, a, [])

def get_comm_tokens(comm, sect_pred=None, suppress_warnings=False):
    '''
    Return list of Tokens in communication, delegating to get_tokens
    for each sentence.
    '''
    return flatten(map(
        lambda sect: flatten(map(
            lambda sent: get_tokens(sent.tokenization, suppress_warnings),
            sect.sentenceList
        )),
        filter(sect_pred, comm.sectionList)
            if (sect_pred is not None)
            else comm.sectionList
    ))



#!/usr/bin/env python

"""Convert Tweet file to Concrete Communications file
"""

import argparse
import codecs
import gzip
import logging
import mimetypes

from concrete.util import CommunicationReader, CommunicationWriter
from concrete.util.twitter import json_tweet_string_to_Communication


def main():
    logging.basicConfig(format='%(levelname)7s:  %(message)s', level=logging.ERROR)

    parser = argparse.ArgumentParser(description="")
    parser.add_argument('tweet_file')
    parser.add_argument('concrete_file')
    args = parser.parse_args()

    if mimetypes.guess_type(args.tweet_file)[1] == 'gzip':
        gz_file = gzip.open(args.tweet_file, 'r')
        utf8_reader = codecs.getreader("utf-8")
        tweet_reader = utf8_reader(gz_file)
    else:
        tweet_reader = codecs.open(args.tweet_file, 'r', encoding='utf-8')

    comm_writer = CommunicationWriter()
    comm_writer.open(args.concrete_file)

    for tweet in tweet_reader:
        tweet = tweet.strip()

        # Ignore empty lines and deleted Tweets
        if tweet and u'{"delete":{"status":' not in tweet:
            comm = json_tweet_string_to_Communication(tweet)
            comm_writer.write(comm)

    comm_writer.close()


if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""
Command line script to (partially) validate a Concrete Communication

This script is a thin wrapper around the functionality in the
concrete.validate library.
"""

import argparse
import logging

import networkx as nx

import concrete.inspect
from concrete.util import CommunicationReader

def main():
    parser = argparse.ArgumentParser(description="Inspect empty dependency parses")
    parser.add_argument('communication_file')
    args = parser.parse_args()

    logging.basicConfig(format='%(levelname)7s:  %(message)s', level=logging.INFO)

    for (comm, filename) in CommunicationReader(args.communication_file):
        logging.info(u"Inspecting Communication with ID '%s" % comm.id)
        for tokenization in concrete.inspect.get_tokenizations(comm):
            inspect_dependency_parses(tokenization)


def inspect_dependency_parses(tokenization):
    def _get_token_text(tokenization):
        return " ".join([t.text for t in tokenization.tokenList.tokenList])

    if tokenization.dependencyParseList:
        total_tokens = len(tokenization.tokenList.tokenList)
        for dependencyParse in tokenization.dependencyParseList:
            dependency_parse_tree = nx.DiGraph()

            # Add nodes to dependency parse tree
            for dependency in dependencyParse.dependencyList:
                if dependency.gov is None and dependency.edgeType.lower() != "root":
                    logging.error(u"  Found a null dependency parse node with governer whose edgeType is '%s' instead of 'root'" %
                                  dependency.edgeType)
                if dependency.gov is not None:
                    if dependency.gov < -1 or dependency.gov > total_tokens:
                        logging.error(u"  Found a null dependency parse node with invalid governer of '%d'" %
                                      dependency.gov)
                    dependency_parse_tree.add_node(dependency.gov)
                dependency_parse_tree.add_node(dependency.dep)

            # Add edges to dependency parse tree
            for dependency in dependencyParse.dependencyList:
                if dependency.gov is not None:
                    dependency_parse_tree.add_edge(dependency.gov, dependency.dep)

            # Check if dependency parse "tree" is a fully connected graph
            undirected_graph = dependency_parse_tree.to_undirected()
            try:
                if not nx.is_connected(undirected_graph):
                    logging.error(u"  The dependency parse graph created by '%s' is not a fully connected graph - the graph has %d components. Token text: '%s'" %
                                  (dependencyParse.metadata.tool, 
                                   nx.number_connected_components(undirected_graph),
                                   _get_token_text(tokenization)))
            except nx.exception.NetworkXPointlessConcept:
                logging.warning(u"  The dependency parse graph created by '%s' does not have any nodes. Token text: '%s'" %
                                (dependencyParse.metadata.tool, _get_token_text(tokenization)))


if __name__ == "__main__":
    main()



"""Convert between JSON and Concrete representations of Tweets

The fields used by the Twitter API are documented at:

  https://dev.twitter.com/overview/api/tweets
"""

import json
import logging
import re
import time

from concrete import (
    AnnotationMetadata,
    BoundingBox,
    Communication,
    CommunicationMetadata,
    HashTag,
    PlaceAttributes,
    TweetInfo,
    TwitterCoordinates,
    TwitterEntities,
    TwitterLatLong,
    TwitterPlace,
    TwitterUser,
    URL,
    UserMention
)

from concrete.util import generate_UUID


TOOL_NAME = "Python module concrete.util.twitter"
TWEET_TYPE = "Tweet"

def json_tweet_object_to_Communication(tweet):
    """
    """
    tweet_info = json_tweet_object_to_TweetInfo(tweet)

    comm = Communication(
        communicationMetadata=CommunicationMetadata(
            tweetInfo=tweet_info),
        metadata=AnnotationMetadata(
            tool=TOOL_NAME,
            timestamp=int(time.time())),
        originalText=tweet_info.text,
        type=TWEET_TYPE,
        uuid=generate_UUID()
    )
    return comm


def json_tweet_object_to_TweetInfo(tweet):
    """
    Args:

    Returns:
    """
    def snake_case_to_camelcase(value):
        """Implementation copied from:

             http://stackoverflow.com/questions/4303492/how-can-i-simplify-this-conversion-from-underscore-to-camelcase-in-python
        """
        def camelcase():
            yield unicode.lower
            while True:
                yield unicode.capitalize
        c = camelcase()
        return u"".join(c.next()(x) if x else u'_' for x in value.split(u"_"))

    def set_flat_fields(concrete_object, twitter_dict):
        """Copy data from the dictionary for the Twitter object to the
        corresponding Concrete data structure, ignoring nested
        data structures.

        The Twitter API uses snake_case for field names while the Concrete
        schema uses CamelCase for the same fields.
        """
        for key in twitter_dict.keys():
            if type(twitter_dict[key]) != dict:
                camelcased_key = snake_case_to_camelcase(key)
                if hasattr(concrete_object, camelcased_key):
                    setattr(concrete_object, camelcased_key, twitter_dict[key])
                else:
                    logging.warn("Concrete schema for '%s' missing field for Twitter API field '%s'" %
                                 (type(concrete_object), key))

    tweet_info = TweetInfo()
    set_flat_fields(tweet_info, tweet)

    twitter_user = TwitterUser()
    set_flat_fields(twitter_user, tweet[u'user'])
    tweet_info.user = twitter_user

    if tweet[u'entities']:
        twitter_entities = TwitterEntities()
        if tweet[u'entities'][u'hashtags']:
            twitter_entities.hashtagList = []
            for hashtag_dict in tweet[u'entities'][u'hashtags']:
                hashtag = HashTag()
                set_flat_fields(hashtag, hashtag_dict)
                twitter_entities.hashtagList.append(hashtag)
        if tweet[u'entities'][u'urls']:
            twitter_entities.urlList = []
            for url_dict in tweet[u'entities'][u'urls']:
                url = URL()
                set_flat_fields(url, url_dict)
                twitter_entities.urlList.append(url)
        if tweet[u'entities'][u'user_mentions']:
            twitter_entities.userMentionList = []
            for user_mention_dict in tweet[u'entities'][u'user_mentions']:
                user_mention = UserMention()
                set_flat_fields(user_mention, user_mention_dict)
                twitter_entities.userMentionList.append(user_mention)
        tweet_info.entities = twitter_entities

        if tweet[u'coordinates']:
            tweet_coordinates = TwitterCoordinates()
            tweet_coordinates.type = tweet[u'coordinates']['type']
            tweet_coordinates.coordinates = TwitterLatLong(
                latitude=tweet[u'coordinates'][u'coordinates'][0],
                longitude=tweet[u'coordinates'][u'coordinates'][1])
            tweet_info.coordinates = tweet_coordinates

        if tweet[u'place']:
            twitter_place = TwitterPlace()
            set_flat_fields(twitter_place, tweet[u'place'])
            if tweet[u'place'][u'bounding_box']:
                bounding_box = BoundingBox()
                set_flat_fields(bounding_box, tweet[u'place'][u'bounding_box'])
                twitter_place.boundingBox = bounding_box
            if tweet[u'place'][u'attributes']:
                place_attributes = PlaceAttributes()
                set_flat_fields(place_attributes, tweet[u'place'][u'attributes'])
                twitter_place.attributes = place_attributes
            tweet_info.place = twitter_place

    return tweet_info


def json_tweet_string_to_Communication(json_tweet_string):
    tweet = json.loads(json_tweet_string)
    return json_tweet_object_to_Communication(tweet)


def json_tweet_string_to_TweetInfo(json_tweet_string):
    tweet = json.loads(json_tweet_string)
    return json_tweet_object_to_TweetInfo(tweet)
